module AssemblyParser where

import Char
import List

import Lexer
import Instruction

assemblyParse = assemblyParseInstruction . removeNewLine .  assemblyRemoveLineComment . removeComment . assemblyFirstLex

assemblyRemoveLineComment = removeTokensBetween (\x -> any (x==) [(TokenSymbol "#"), (TokenSymbol ";")]) (==TokenNewLine) 0 

assemblyFirstLex [] = []
assemblyFirstLex s =
    if fst alphas /= "" then
        TokenIdentifier (fst alphas) : assemblyFirstLex (snd alphas)
    else if fst newline /= "" then
             TokenNewLine : assemblyFirstLex (snd newline)
         else if fst symbols /= "" then
                  TokenSymbol (fst symbols) : assemblyFirstLex (snd symbols)
              else if fst symbol /= "" then
                       TokenSymbol (fst symbol) : assemblyFirstLex (snd symbol)
                   else
                       assemblyFirstLex (tail s)
    where
      alwaysFalse x = False
      alphas = tokenerWord fp fp s ""
          where
            fp x = isAlphaNum x || isIdSymbols x
      newline = tokenerWord isNewLine alwaysFalse s ""
          where
            isNewLine x = any (x==) "\r\n"
      symbol = tokenerWord fp alwaysFalse s ""
          where
            fp x = ((not.isAlphaNum) x) && (not.isSpace) x && not (isIdSymbols x)
      symbols = findSymbols multiSymbols s
      isIdSymbols x = any (==x) ['+', '-', '_', '.']
      multiSymbols = ["//", "/*", "*/"]
      findSymbols (sym:syms) cs | length cs >= length sym && sym == take (length sym) cs = (sym, (tails cs)!!(length sym))
      findSymbols (sym:syms) cs = findSymbols (syms) cs
      findSymbols [] cs = ("", cs)

upperString = map toUpper

assemblyParseInstruction ((TokenIdentifier ins):tokens)
    | any ((upperString ins)==) ["KA", "AO", "CH", "CY", "AM", "MA", "M+", "M-"] =
        (Operand0 (upperString ins), ""): assemblyParseInstruction tokens

assemblyParseInstruction ((TokenIdentifier ins):(TokenIdentifier op):tokens)
    | any ((upperString ins)==) ["TIA", "AIA", "TIY", "AIY", "CIA", "CIY", ".NIBBLE"] =
        (OperandInt1 (upperString ins) (readHexChar (upperString op)), ""): assemblyParseInstruction tokens

assemblyParseInstruction ((TokenIdentifier ins):(TokenIdentifier op):tokens)
    | any ((upperString ins)==) ["CAL", ".ALIGN"] =
        (OperandString1 (upperString ins) (upperString op), ""): assemblyParseInstruction tokens

assemblyParseInstruction ((TokenIdentifier ins):(TokenIdentifier op):tokens)
    | any ((upperString ins)==) ["JUMP"] =
        (OperandString1 (upperString ins) op, ""): assemblyParseInstruction tokens

assemblyParseInstruction ((TokenIdentifier label):(TokenSymbol ":"):tokens) =
    (Label label, ""): assemblyParseInstruction tokens

assemblyParseInstruction (s:tokens) = lexError ("unkown instruction " ++ show s)

assemblyParseInstruction [] = []
